/*
 * BasicRationNumber.java
 *
 * Copyright (C) 1998  Utrecht University, Netherlands
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package nl.coffee.math;

/**
 * Basic implementation of a rational number, i.e. a fraction.
 *
 * <p>This implementation simply encapsulates a <em>numerator</em> and
 * a <em>denominator</em> field. Both are <code>IntegerNumber</code>
 * objects.
 *
 * @version 1.1 199809.24
 * @author Ernst de Haan
 *
 * @since JUMP_1.0
 */
public class BasicRationalNumber
   extends AbstractRationalNumber
{
   //=========================================================================
   // CONSTRUCTORS
   //=========================================================================

   /**
    * Constructs a new <code>BasicRationalNumber</code> from a numerator and
    * a denominator.
    *
    * @param numerator
    *    the numerator for the fraction.
    *
    * @param denominator
    *    the denominator for the fraction.
    *
    * @exception IllegalArgumentException
    *    if one of the following applies:
    *    <ol>
    *       <li><em>numerator</em>   == <code>null</code></li>
    *       <li><em>denominator</em> == <code>null</code></li>
    *    </ol>
    *
    * @exception ArithmeticException
    *    if the value of <em>denominator</em> is zero.
    */
   protected BasicRationalNumber(IntegerNumber numerator,
                                 IntegerNumber denominator)
      throws IllegalArgumentException,
             ArithmeticException
   {
      ExceptionSupport.checkNumeratorNotNull  (numerator);
      ExceptionSupport.checkDenominatorNotNull(denominator);

      //----------------------------------------------------
      // NOTE: normalize() will throw ArithmeticException if
      //       denominator is null.
      //----------------------------------------------------

      ExceptionSupport.checkDivideByZero(denominator);

      IntegerNumber divisor = numerator.gcd(denominator);

      numerator   = numerator.integerDivide(divisor);
      denominator = denominator.integerDivide(divisor);

      if (denominator.getSign() == -1)
      {
         numerator   = (IntegerNumber) numerator.negate();
         denominator = (IntegerNumber) denominator.negate();
      }

      this.numerator   = numerator;
      this.denominator = denominator;
   }


   //=========================================================================
   // FACTORY METHODS:
   //
   // - createInstance
   //
   //=========================================================================

   /**
    * Returns an instance of a <code>BasicRationalNumber</code> with the
    * given numerator and denominator.
    *
    * @param numerator
    *    the numerator for the fraction.
    *
    * @param denominator
    *    the denominator for the fraction.
    *
    * @return
    *    the (possibly newly constructed) instance.
    *
    * @exception IllegalArgumentException
    *    if one of the following applies:
    *    <ol>
    *       <li><em>numerator</em>   == <code>null</code></li>
    *       <li><em>denominator</em> == <code>null</code></li>
    *    </ol>
    *
    * @exception ArithmeticException
    *    if the value of <code>denominator</code> is zero.
    */
   public static BasicRationalNumber createInstance(IntegerNumber numerator,
                                                    IntegerNumber denominator)
      throws IllegalArgumentException,
             ArithmeticException
   {
      ExceptionSupport.checkNumeratorNotNull  (numerator);
      ExceptionSupport.checkDenominatorNotNull(denominator);

      if (denominator.compareTo(LongIntegerNumber.TWO) == 0)
      {
         if (numerator.compareTo(LongIntegerNumber.ONE) == 0)
         {
            return ONE_HALF;
         }
         if (numerator.compareTo(LongIntegerNumber.MINUS_ONE) == 0)
         {
            return MINUS_ONE_HALF;
         }
      }

      //------------------------------------------------
      // NOTE: constructor may throw ArithmeticException
      //------------------------------------------------

      return new BasicRationalNumber(numerator, denominator);
   }


   //=========================================================================
   // CACHED INSTANCES
   //
   // - ONE_HALF
   // - MINUS_ONE_HALF
   //
   //=========================================================================

   /**
    * Cached instance that represents the number one half, 1/2.
    */
   public static final BasicRationalNumber ONE_HALF =
      new BasicRationalNumber(LongIntegerNumber.ONE,
                              LongIntegerNumber.TWO);

   /**
    * Cached instance that represents the number minus one half, -1/2.
    */
   public static final BasicRationalNumber MINUS_ONE_HALF =
      new BasicRationalNumber(LongIntegerNumber.MINUS_ONE,
                              LongIntegerNumber.TWO);


   //=========================================================================
   // FRACTION PROPERTIES:
   //
   // - numerator
   // - denominator
   //
   //=========================================================================

   /**
    * The numerator of this fraction.
    */
   public final IntegerNumber numerator;

   public IntegerNumber getNumerator()
   {
      return numerator;
   }

   /**
    * The denominator of this fraction.
    */
   public final IntegerNumber denominator;

   public IntegerNumber getDenominator()
   {
      return denominator;
   }

}
