/*
 * IntegerNumber.java
 *
 * Copyright (C) 1998  Utrecht University, Netherlands
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package nl.coffee.math;

import java.math.*;

/**
 * An immutable integer number. This interface provides the base for all
 * integer numbers. It extends <code>RationalNumber</code> interface by
 * offering narrowed numeric computations for integer numbers.
 *
 * <p>Some <code>IntegerNumber</code> numeric computations are narrowed
 * compared to their more general <code>RationalNumber</code> equivalents.
 * For instance, adding two integer numbers will result in a integer number.
 * The computations concerned are:
 *
 * <ul>
 *    <li><code>add(IntegerNumber)</code> --
 *              adding two integer numbers will result in another integer
 *              number</li>
 *
 *    <li><code>subtract(RationalNumber)</code> --
 *              subtracting one rational number from another will result in an
 *              integer number</li>
 *
 *    <li><code>multiply(RationalNumber)</code> --
 *              multiplying two integer numbers will result in another integer
 *              number</li>
 *
 * </ul>
 *
 * @version 1.1 199809.24
 * @author Ernst de Haan
 *
 * @since JUMP_1.0
 */
public interface IntegerNumber
   extends RationalNumber
{
   //=========================================================================
   // UNARY OPERATIONS:
   //
   // - abs
   // - negate
   //
   // --------------------------------------------------------------
   // Not actually defined, because the Java language does not (yet)
   // support narrowed return types in overloaded methods.
   // --------------------------------------------------------------
   //
   //=========================================================================

//   /**
//    * Computes |this|.
//    *
//    * @return the result.
//    */
//   public IntegerNumber abs();

//   /**
//    * Computes -this.
//    *
//    * @return the result.
//    */
//   public IntegerNumber negate();


   //=========================================================================
   // COMPARISON:
   //
   // - compareTo
   //
   //=========================================================================

   public int compareTo(IntegerNumber n)
      throws IllegalArgumentException;


   //=========================================================================
   // BINARY OPERATIONS:
   //
   // - add
   // - subtract
   // - multiply
   //
   //=========================================================================

   /**
    * Computes this+<em>n</em>, where <em>n</em> is an integer number.
    *
    * @param n the number to add to this.
    * @return the sum of this and <em>n</em>.
    * @exception IllegalArgumentException if the argument is
    *    <code>null</code>.
    */
   public IntegerNumber add(IntegerNumber n)
      throws IllegalArgumentException;

   /**
    * Computes this-<em>n</em>, where <em>n</em> is an integer number.
    *
    * @param n the number to subtract from this.
    * @return this minus <em>n</em>.
    * @exception IllegalArgumentException if the argument is
    *    <code>null</code>.
    */
   public IntegerNumber subtract(IntegerNumber n)
      throws IllegalArgumentException;

   /**
    * Computes this*<em>n</em>, where <em>n</em> is an integer number.
    *
    * @param n the number to multiply this by.
    * @return the product of this and <em>n</em>.
    * @exception IllegalArgumentException if the argument is
    *    <code>null</code>.
    */
   public IntegerNumber multiply(IntegerNumber n)
      throws IllegalArgumentException;


   //=========================================================================
   // UNARY INTEGER OPERATIONS:
   //
   // - dec
   // - inc
   // - fac
   //=========================================================================

   /**
    * Computes this-1.
    *
    * @return this decreased by 1.
    */
   public IntegerNumber dec();

   /**
    * Computes this+1.
    *
    * @return this increased by 1.
    */
   public IntegerNumber inc();

   /**
    * Computes this!. The value of this cannot be negative.
    *
    * @return the faculty of this.
    * @exception IllegalArgumentException if the value of this &lt; 0.
    */
   public IntegerNumber fac()
      throws IllegalArgumentException;


   //=========================================================================
   // BINARY INTEGER OPERATIONS:
   //
   // - integerDivide
   // - remainder
   // - gcd
   // - isRelativePrime
   //
   //=========================================================================

   /**
    * Computes the integer result of this/<em>n</em>, where <em>n</em> is an
    * integer number.
    *
    * @param n the number to divide this by.
    * @return the integer result of this divided by <em>n</em>.
    * @exception ArithmeticException if the value of the argument number is
    *    zero.
    * @exception IllegalArgumentException if the argument is
    *    <code>null</code>.
    */
   public IntegerNumber integerDivide(IntegerNumber n)
      throws ArithmeticException, IllegalArgumentException;

   /**
    * Computes the remainder of this/<em>n</em>, where <em>n</em> is an
    * integer number.
    *
    * @param n the number to divide this by.
    * @return the remainder.
    * @exception ArithmeticException if the value of the argument number is
    *    zero.
    * @exception IllegalArgumentException if the argument is
    *    <code>null</code>.
    */
   public IntegerNumber remainder(IntegerNumber n)
      throws ArithmeticException, IllegalArgumentException;

   /**
    * Computes the greatest common divisor of this and <em>n</em>.
    *
    * @param n
    *    the other integer number.
    *
    * @return
    *    the greatest common divisor.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> is <code>null</code>.
    */
   public IntegerNumber gcd(IntegerNumber n)
      throws IllegalArgumentException;

   /**
    * Determines if this and <em>n</em> are relative primes.
    *
    * @param n
    *    the other integer number.
    *
    * @return
    *    <code>true</code> if this and <em>n</em> are relative primes,
    *    <code>false</code> otherwise.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> is <code>null</code>.
    */
   public boolean isRelativePrime(IntegerNumber n)
      throws IllegalArgumentException;


   //=========================================================================
   // NUMERIC INTEGER CONVERSION:
   //
   // - toByteArray
   // - toBigInteger
   // - toBigDecimal
   //
   //=========================================================================

   /**
    * Returns the two's-complement representation of this integer number.
    * The array is big-endian (i.e., the most significant byte is in the
    * [0] position). The array contains the minimum number of bytes required
    * to represent the number.
    *
    * @return a byte array containing the bits of this integer number.
    */
   public byte[] toByteArray();

   /**
    * Converts the value of this number to a <code>BigInteger</code>.
    *
    * @return a <code>BigInteger</code> with the exact value of this.
    */
   public BigInteger toBigInteger();

   /**
    * Converts the value of this number to a <code>BigDecimal</code>. A
    * precision nor a rounding mode need to be specified for integer numbers.
    *
    * @return a <code>BigDecimal</code> with the exact value of this.
    */
   public BigDecimal toBigDecimal();

}
