/*
 * RationalNumber.java
 *
 * Copyright (C) 1998  Utrecht University, Netherlands
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package nl.coffee.math;

/**
 * An immutable rational number. This interface provides the base for all
 * rational numbers. It extends the <code>RealNumber</code> interface by
 * offering narrowed numeric computations for rational numbers.
 *
 * <p>All rational values can be written as a fraction
 * <em>x</em>/<em>y</em>, where both <em>x</em> and <em>y</em> are
 * integer numbers.
 *
 * <p><code>RationalNumber</code> derivates must obey these rules:
 *
 * <ul>
 *    <li><em>they should be normalized</em> --
 *        the fraction properties should allways return a fraction that
 *        cannot be simplified any further, thus 3/9 is illegal,
 *        while 1/3 is legal</li>
 *
 *    <li><em>the denominator should allways be positive</em> --
 *        if the fraction is negative, the numerator should be negative
 *        and the denominator positive, otherwise both should be positive,
 *        thus both -1/-3 and 1/-3 are illegal, while 1/3 and -1/3
 *        are legal</li>
 *
 * </ul>
 *
 * <p>Some <code>RationalNumber</code> numeric computations are narrowed
 * compared to their more general <code>RealNumber</code> equivalents. For
 * instance, adding two rational numbers will result in a rational number.
 * The computations concerned are:
 *
 * <ul>
 *    <li><code>add(RationalNumber)</code> --
 *        adding two rational numbers
 *        will result in another rational number</li>
 *
 *    <li><code>subtract(RationalNumber)</code> --
 *        subtracting one rational number from another
 *        will result in a rational number</li>
 *
 *    <li><code>multiply(RationalNumber)</code> --
 *        multiplying two rational numbers
 *        will result in another rational number</li>
 *
 *    <li><code>divide(RationalNumber)</code> --
 *        dividing one rational number by another
 *        will result in a rational number</li>
 *
 *    <li><code>pow(IntegerNumber)</code> --
 *        raising a rational number to an integer power
 *        will result in a rational number</li>
 *
 * </ul>
 *
 * @version 1.1 199809.24
 * @author Ernst de Haan
 *
 * @since JUMP_1.0
 */
public interface RationalNumber extends RealNumber
{
   //=========================================================================
   // UNARY OPERATIONS:
   //
   // - abs
   // - negate
   // - invert
   //
   // -----------------------------------------------------------------
   // Not actually narrowed, because the Java language does not (yet)
   // support narrowed return types in overloaded methods.
   // -----------------------------------------------------------------
   //
   //=========================================================================

//   /**
//    * Computes |this|
//    *
//    * @return
//    *    the result.
//    */
//   public RationalNumber abs();

//   /**
//    * Computes -this.
//    *
//    * @return
//    *    the result.
//    */
//   public RationalNumber negate();

//   /**
//    * Computes 1/this.
//    *
//    * @return
//    *    the result.
//    *
//    * @exception ArithmeticException
//    *    if the value of this is zero.
//    */
//   public RationalNumber invert()
//      throws ArithmeticException;


   //=========================================================================
   // COMPARISON
   //=========================================================================

   public int compareTo(RationalNumber n)
      throws IllegalArgumentException;


   //=========================================================================
   // BINARY OPERATIONS:
   //
   // - add
   // - subtract
   // - multiply
   // - divide
   // - pow
   //
   //=========================================================================

   /**
    * Computes this+<em>n</em>, where <em>n</em> is a rational number.
    *
    * @param n
    *    the number to add to this.
    *
    * @return
    *    the sum of this and <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> == <code>null</code>.
    */
   public RationalNumber add(RationalNumber n)
      throws IllegalArgumentException;

   /**
    * Computes this-<em>n</em>, where <em>n</em> is a rational number.
    *
    * @param n
    *    the number to subtract from this.
    *
    * @return
    *    this minus <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> == <code>null</code>.
    */
   public RationalNumber subtract(RationalNumber n)
      throws IllegalArgumentException;

   /**
    * Computes this*<em>n</em>, where <em>n</em> is a rational number.
    *
    * @param n
    *    the number to multiply this by.
    *
    * @return
    *    the product of this and <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> == <code>null</code>.
    */
   public RationalNumber multiply(RationalNumber n)
      throws IllegalArgumentException;

   /**
    * Computes this/<em>n</em>, where <em>n</em> is a rational number.
    *
    * @param n
    *    the number to divide this by.
    *
    * @return
    *    this divided by <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> == <code>null</code>.
    *
    * @exception ArithmeticException
    *    if the value of <em>n</em> is zero.
    */
   public RationalNumber divide(RationalNumber n)
      throws IllegalArgumentException,
             ArithmeticException;

   /**
    * Computes this**<em>n</em>, where <em>n</em> is an integer number.
    *
    * @param n
    *    the exponent.
    *
    * @return
    *    this raised to the power of <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> == <code>null</code>.
    */
   public RationalNumber pow(IntegerNumber n);


   //=========================================================================
   // FRACTION PARTS
   //=========================================================================

   /**
    * Returns the numerator of this fraction. This numerator may be negative.
    *
    * @return
    *    the numerator.
    */
   public IntegerNumber getNumerator();

   /**
    * Returns the denominator of this fraction. The denonimator will allways
    * be a positive integer number.
    *
    * @return
    *    the denominator.
    */
   public IntegerNumber getDenominator();


}
