/*
 * RealNumber.java
 *
 * Copyright (C) 1998  Utrecht University, Netherlands
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package nl.coffee.math;

import java.math.*;

/**
 * An immutable real number. This interface provides the base for all real
 * numbers, with a number of basic numeric operations, like:
 *
 * <ul>
 *    <li><code>negate()</code> --
 *        Computes -this</li>
 *
 *    <li><code>invert()</code> --
 *        Computes 1/this</li>
 *
 *    <li><code>add(RealNumber)</code> --
 *        Computes this+<em>n</em></li>
 *
 *    <li><code>subtract(RealNumber)</code> --
 *        Computes this-<em>n</em></li>
 *
 *    <li><code>multiply(RealNumber)</code> --
 *        Computes this*<em>n</em></li>
 *
 *    <li><code>divide(RealNumber)</code> --
 *        Computes this/<em>n</em></li>
 *
 *    <li><code>pow(RealNumber)</code> --
 *        Computes this**<sup><em>n</em></sup></li>
 *
 * </ul>
 *
 * <p>Examples of real numbers are 3, -1.68, the square root of 3, 2/3,
 * <em>pi</em>, 18.2**2.3 and <em>e</em>.
 *
 * <p>Real numbers can be cast to native Java numbers with these conversion
 * methods:
 *
 * <ul>
 *    <li><code>longValue()</code> --
 *        Converts this number to a <code>long</code>.
 *        Use <code>fitsLong()</code> to make sure
 *        the conversion will succeed</li>
 *
 *    <li><code>intValue()</code> --
 *        Converts this number to an <code>int</code>.
 *        Use <code>fitsInt()</code> to make sure
 *        the conversion will succeed</li>
 *
 *    <li><code>shortValue()</code> --
 *        Converts this number to a <code>short</code>. Use
 *        <code>fitsShort()</code> to make sure
 *        the conversion will succeed</li>
 *
 *    <li><code>byteValue()</code> --
 *        Converts this number to a <code>byte</code>.
 *        Use <code>fitsByte()</code> to make sure
 *        the conversion will succeed</li>
 *
 *    <li><code>doubleValue()</code> --
 *        Converts this number to a <code>double</code>.
 *        Use <code>fitsDouble()</code> to make sure
 *        the conversion will succeed</li>
 *
 *    <li><code>floatValue()</code> --
 *        Converts this number to a <code>float</code>.
 *        Use <code>fitsFloat()</code> to make sure
 *        the conversion will succeed</li>
 *
 * </ul>
 *
 * <p>Use the <code>NumberCentral</code> to obtain <code>RealNumber</code>
 * instances. For instance:
 *
 * <blockquote><code>RealNumber n =
 *    NumberCentral.valueOf(0.399);</code></blockquote>
 *
 * <p>Concrete implementations of this interface should implement
 * the <code>boolean equals(Object)</code> method derived from
 * <code>java.lang.Object</code> in order to compare <code>RealNumber</code>
 * instances for equality.
 *
 * <p>PENDING (1): The names of the following methods may be changed in the
 * near future to reflect the expected change in the syntax of the Java
 * language syntax to include operator overloading based on method names:
 *
 * <ul>
 *    <li><em>negate</em>   --&gt; <em>uminus</em></li>
 *    <li><em>add</em>      --&gt; <em>plus</em></li>
 *    <li><em>subtract</em> --&gt; <em>minus</em></li>
 *    <li><em>multiply</em> --&gt; <em>times</em></li>
 * </ul>
 *
 * <p>PENDING (2): Rounding modes different from <em>trunc</em> should be
 * specified. Method names may include <em>roundUp</em>, <em>roundDown</em>
 * etc. if this is in accordance with proposed changes to language syntax
 * (See: "The Evolution of Numerical Computing in Java" by James Gosling,
 * available from the Sun JavaSoft website.)
 *
 * @see NumberCentral
 *
 * @version 1.2 199809.24
 * @author Ernst de Haan
 *
 * @since JUMP_1.0
 */
public interface RealNumber
{
   //=========================================================================
   // PROPERTY: sign
   //
   // - getSign
   //
   //=========================================================================

   /**
    * Determines the sign of this number. This method returns -1, 0, or
    * 1 as this number is negative, zero, or positive.
    *
    * @return
    *    the sign of this number.
    */
   public int getSign();


   //=========================================================================
   // COMPARISON:
   //
   // - compareTo
   // - retryCompare
   //
   //=========================================================================

   /**
    * Returns -1, 0 or 1 as this number is less than, equal to, or greater
    * than <em>n</em>, where <em>n</em> is a real number. If this number
    * cannot be compared with <em>n</em>, then
    * <code>n.retryCompare(this)</code> should be called in order to try
    * to let <em>n</em> compare to this. If that succeeds, the negative of
    * the result may be returned. If fails, it will throw a
    * <code>CanNotCompareException</code>.
    *
    * @param n
    *    the number to compare to.
    *
    * @return
    *    -1 if this &lt; <em>n</em>,
    *     0 if this ==   <em>n</em>,
    *     1 if this &gt; <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> == <code>null</code>.
    *
    * @exception CanNotCompareException
    *    if this number can not compare with the <em>n</em>, and vice versa.
    */
   public int compareTo(RealNumber n)
      throws IllegalArgumentException,
             CanNotCompareException;

   /**
    * Returns -1, 0 or 1 as this number is less than, equal to, or greater
    * than <em>n</em>, where <em>n</em> is a real number. If this number
    * cannot be compared with <em>n</em>, then a
    * <code>CanNotCompareException</code> is thrown.
    *
    * @param n
    *    the number to compare to.
    *
    * @return
    *    -1 if this &lt; <em>n</em>,
    *     0 if this ==   <em>n</em>,
    *     1 if this &gt; <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> == <code>null</code>.
    *
    * @exception CanNotCompareException
    *    if this number can not compare with the <em>n</em>.
    */
   public int retryCompare(RealNumber n)
      throws IllegalArgumentException,
             CanNotCompareException;


   //=========================================================================
   // UNARY OPERATIONS:
   //
   // - abs
   // - negate
   // - invert
   //
   //=========================================================================

   /**
    * Computes |this|. The returned value is allways &gt;= 0.
    *
    * @return
    *    the absolute of this.
    */
   public RealNumber abs();

   /**
    * Computes -this.
    *
    * @return
    *    the negative of this.
    */
   public RealNumber negate();

   /**
    * Computes 1/this.
    *
    * @return
    *    the inverse of this.
    *
    * @exception ArithmeticException
    *    if the value of this is zero.
    */
   public RealNumber invert()
      throws ArithmeticException;


   //=========================================================================
   // BINARY OPERATIONS:
   //
   // - add
   // - subtract
   // - multiply
   // - divide
   // - pow
   //
   //=========================================================================

   /**
    * Computes this+<em>n</em>, where <em>n</em> is a real number.
    *
    * @param n
    *    the number to add to this.
    *
    * @return
    *    the sum of this and <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> is <code>null</code>.
    */
   public RealNumber add(RealNumber n)
      throws IllegalArgumentException;

   /**
    * Computes this-<em>n</em>, where <em>n</em> is a real number.
    *
    * @param n
    *    the number to subtract from this.
    *
    * @return
    *    this minus <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> is <code>null</code>.
    */
   public RealNumber subtract(RealNumber n)
      throws IllegalArgumentException;

   /**
    * Computes this*<em>n</em>, where <em>n</em> is a real number.
    *
    * @param n
    *    the factor, the number to multiply this by.
    *
    * @return
    *    the product of this and <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> is <code>null</code>.
    */
   public RealNumber multiply(RealNumber n)
      throws IllegalArgumentException;

   /**
    * Computes this/<em>n</em>, where <em>n</em> is a real number.
    *
    * @param n
    *    the number to divide this by.
    *
    * @return
    *    this divided by <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> is <code>null</code>.
    *
    * @exception ArithmeticException
    *    if the value of <em>n</em> is zero.
    */
   public RealNumber divide(RealNumber n)
      throws IllegalArgumentException,
             ArithmeticException;

   /**
    * Computes this**<em>n</em>, where <em>n</em> is a real number.
    *
    * @param n
    *    the exponent.
    *
    * @return
    *    this raised to the power of <em>n</em>.
    *
    * @exception IllegalArgumentException
    *    if <em>n</em> is <code>null</code>.
    */
   public RealNumber pow(RealNumber n)
      throws IllegalArgumentException;


   //=========================================================================
   // NUMERIC CONVERSION FIT CHECKING:
   //
   // - fitsLong
   // - fitsInt
   // - fitsShort
   // - fitsByte
   // - fitsDouble
   // - fitsFloat
   //
   //=========================================================================

   /**
    * Determines if the truncated value of this number fits in a
    * <code>long</code>.
    *
    * @return
    *    <code>true</code> iff this value fits in a <code>long</code>.
    */
   public boolean fitsLong();

   /**
    * Determines if the truncated value of this number fits in an
    * <code>int</code>.
    *
    * @return
    *    <code>true</code> iff this value fits in an <code>int</code>.
    */
   public boolean fitsInt();

   /**
    * Determines if the truncated value of this number fits in a
    * <code>short</code>.
    *
    * @return
    *    <code>true</code> iff this value fits in a <code>short</code>.
    */
   public boolean fitsShort();

   /**
    * Determines if the truncated value of this number fits in a
    * <code>byte</code>.
    *
    * @return
    *    <code>true</code> iff this value fits in a <code>byte</code>.
    */
   public boolean fitsByte();

   /**
    * Determines if the rounded value of this number fits in a
    * <code>double</code>.
    *
    * @return
    *    <code>true</code> iff this value fits in a <code>double</code>.
    */
   public boolean fitsDouble();

   /**
    * Determines if the rounded value of this number fits in a
    * <code>float</code>.
    *
    * @return
    *    <code>true</code> iff this value fits in a <code>float</code>.
    */
   public boolean fitsFloat();


   //=========================================================================
   // NUMERIC CONVERSION:
   //
   // - longValue
   // - intValue
   // - shortValue
   // - byteValue
   // - doubleValue
   // - floatValue
   // - toBigDecimal
   // - toBigInteger
   //
   //=========================================================================

   /**
    * Returns the value of this number as a <code>long</code>. This may
    * involve rounding.
    *
    * @return
    *    the numeric value represented by this object after conversion
    *    to type <code>long</code>.
    */
   public long longValue();

   /**
    * Returns the value of this number as an <code>int</code>. This may
    * involve rounding.
    *
    * @return
    *    the numeric value represented by this object after conversion
    *    to type <code>int</code>.
    */
   public int intValue();

   /**
    * Returns the value of this number as a <code>short</code>. This may
    * involve rounding.
    *
    * @return
    *    the numeric value represented by this object after conversion
    *    to type <code>short</code>.
    */
   public short shortValue();

   /**
    * Returns the value of this number as a <code>byte</code>. This may
    * involve rounding.
    *
    * @return
    *    the numeric value represented by this object after conversion
    *    to type <code>byte</code>.
    */
   public byte byteValue();

   /**
    * Returns the value of this number as a <code>double</code>. This may
    * involve rounding.
    *
    * @return
    *    the numeric value represented by this object after conversion
    *    to type <code>double</code>.
    */
   public double doubleValue();

   /**
    * Returns the value of this number as a <code>float</code>. This may
    * involve rounding.
    *
    * @return
    *    the numeric value represented by this object after conversion
    *    to type <code>float</code>.
    */
   public float floatValue();

   /**
    * Converts the value of this number to a <code>BigDecimal</code> with the
    * specified precision. This method uses the <code>ROUND_HALF_UP</code>
    * rounding mode as defined in <code>BigDecimal</code>.
    *
    * @param precision
    *    the number of digits behind the decimal point.
    *
    * @return
    *    a <code>BigDecimal</code> with the rounded value of this.
    *
    * @exception IllegalArgumentException
    *    if <em>precision</em> &lt; 0.
    */
   public BigDecimal toBigDecimal(int precision)
      throws IllegalArgumentException;

   /**
    * Converts the value of this number to a <code>BigDecimal</code> with the
    * specified precision and rounding mode.
    *
    * @param precision
    *    the number of digits behind the decimal point.
    *
    * @param roundingMode
    *    the rounding mode to use, one of the modes defined
    *    in class <code>BigDecimal</code>.
    *
    * @return
    *    a <code>BigDecimal</code> with the rounded value of this.
    *
    * @exception IllegalArgumentException
    *    if one of the following applies:
    *    <ol>
    *       <li><em>precision</em> &lt; 0</li>
    *       <li><em>roundingMode</em> is not one of the valid rounding modes
    *           defined in class <code>BigDecimal</code></li>
    *    </ol>
    */
   public BigDecimal toBigDecimal(int precision, int roundingMode)
      throws IllegalArgumentException;

   /**
    * Converts the value of this number to a <code>BigInteger</code>. This
    * may involve rounding.
    *
    * @return
    *    the numeric value represented by this object after conversion
    *    to type <code>BigInteger</code>.
    */
   public BigInteger toBigInteger();


   //=========================================================================
   // ROUNDING:
   //
   // - trunc
   //
   //=========================================================================

   /**
    * Rounds to an integer number towards 0.
    *
    * @return
    *    this real number truncated to an integer.
    */
   public IntegerNumber trunc();

}
