/*
 * Sum.java
 *
 * Copyright (C) 1998  Utrecht University, Netherlands
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package nl.coffee.math;

import java.math.*;

/**
 * A sum of two real numbers.
 *
 * <p>Instances of <code>Sum</code> must be obtained by using one of the
 * <code>createInstance()</code> factory methods.
 *
 * @version 1.1 199809.24
 * @author Ernst de Haan
 *
 * @since JUMP_1.0
 */
public class Sum
   extends AbstractCompositeNumber
{
   //=========================================================================
   // CONSTRUCTOR
   //=========================================================================

   /**
    * Constructs a <code>Sum</code> based on the 2 specified operands.
    *
    * @param operand1
    *    the first operand for the sum.
    *
    * @param operand2 the second operand for the sum.
    *
    * @exception IllegalArgumentException
    *    if one of the following applies:
    *    <ol>
    *       <li><em>a</em> == <code>null</code></li>
    *       <li><em>b</em> == <code>null</code></li>
    *    </ol>
    */
   protected Sum(RealNumber a, RealNumber b)
      throws IllegalArgumentException
   {
      ExceptionSupport.checkANotNull(a);
      ExceptionSupport.checkBNotNull(b);

      this.elements = new RealNumber[2];
      elements[0] = a;
      elements[1] = b;

      try
      {
         sign = determineSign(a, b);
      }
      catch (CanNotCompareException e)
      {
         //------------------
         // A T T E N T I O N
         //
         // WORKAROUND / BUG
         //------------------

         sign = 2;
      }
   }


   //=========================================================================
   // FACTORY METHODS:
   //
   // - createInstance (2x)
   //
   //=========================================================================

   /**
    * Returns a <code>Sum</code> with the specified operands.
    *
    * @param a the first operand.
    * @param b the second operand.
    * @return the <code>Sum</code> instance, possibly newly constructed.
    * @exception IllegalArgumentException if either argument is
    *    <code>null</code>.
    */
   public static Sum createInstance(RealNumber a, RealNumber b)
      throws IllegalArgumentException
   {
      return new Sum(a, b);
   }

   //---------------------------
   // PENDING: add this method ?
/*
   public static Sum createInstance(RealNumber[] operands)
      throws IllegalArgumentException
   {
      ExceptionSupport.checkOperandsNotNull(operands);
      ExceptionSupport.checkOperandsLengthAtLeast2(operands);

      if (operands.length == 2)
      {
         return createInstance(operands[0], operands[1]);
      }

      RealNumber[] lastOperands = new RealNumber[operands.length - 1];
      System.arraycopy(lastOperands, 1, operands, 0, operands.length - 1);

      return createInstance(operands[0], createInstance(lastOperands));
   }
*/
   //---------------------------


   //=========================================================================
   // DETERMINE SIGN OF SUM:
   //
   // - determineSign
   //
   //=========================================================================

   /**
    * Computes the sign of a sum with the specified operands.
    *
    * @param operand1 the first operand.
    * @param operand2 the second operand.
    */
   protected static int determineSign(RealNumber a, RealNumber b)
      throws CanNotCompareException
   {
      int signA = a.getSign();
      int signB = b.getSign();

      if (signA != signB)
      {
         if (signA==-1 && signB==1)
         {
            return a.compareTo(b.negate());
         }

         if (signA==1 && signB==-1)
         {
            return b.compareTo(a.negate());
         }

         if (signA==0)
         {
            return signB;
         }

         // implicit else if (signB==0)
         return signA;
      }

      // implicit else (signA==signB)
      return signA;
   }


   //=========================================================================
   // PROPERTY: sign
   //
   // - sign
   // - getSign
   //
   //=========================================================================

   /**
    * The sign of this sum. This field is set only once upon construction.
    */
   protected int sign;

   /**
    * Determines the sign of this number. This method returns -1, 0, or
    * 1 as this number is negative, zero, or positive.
    *
    * @return the sign of this number.
    */
   public int getSign()
   {
      return sign;
   }


   //=========================================================================
   // COMPARISON:
   //
   // - compareTo
   //
   //=========================================================================

   /**
    * Returns -1, 0 or 1 as this number is less than, equal to, or greater
    * than <em>n</em>, where <em>n</em> is a real number.
    *
    * @param n the number to compare to.
    * @return -1 if this&lt;<em>n</em>, 0 if this==<em>n</em>, 1 if
    *    this&gt;<em>n</em>.
    * @exception IllegalArgumentException if the argument is
    *    <code>null</code>.
    */
   public int retryCompare(RealNumber n)
      throws IllegalArgumentException,
             CanNotCompareException
   {
      int thatSign = n.getSign();

      if (sign > thatSign)                { return  1; }
      if (sign < thatSign)                { return -1; }
      if ((sign == 0) && (thatSign == 0)) { return  0; }

      //------------------------------
      // PENDING: How do we do this ?!
      //------------------------------

      throw new CanNotCompareException(n, this);
   }


   //=========================================================================
   // NUMERIC CONVERSION
   //
   // - toBigDecimal
   //
   //=========================================================================

   /**
    * Converts the value of this number to a <code>BigDecimal</code> with the
    * specified precision and rounding mode.
    *
    * @param precision the number of digits behind the decimal point.
    * @param roundingMode the rounding mode to use, one of the modes defined
    *    in class <code>BigDecimal</code>.
    * @return a <code>BigDecimal</code> with the rounded value of this.
    * @exception IllegalArgumentException if <em>precision</em>&lt;0 or
    *    the rounding mode is not one of the valid rounding modes defined in
    *    class <code>BigDecimal</code>.
    */
   public BigDecimal toBigDecimal(int precision, int roundingMode)
      throws IllegalArgumentException
   {
      BigDecimal a = elements[0].toBigDecimal(precision+1,
         BigDecimal.ROUND_HALF_UP);
      BigDecimal b = elements[1].toBigDecimal(precision+1,
         BigDecimal.ROUND_HALF_UP);

      // compute BigDecimal sum
      BigDecimal result = a.add(b);

      // return correct precision
      return result.setScale(precision, roundingMode);
   }


   //=========================================================================
   // ROUNDING:
   //
   // - trunc
   //
   //=========================================================================

   /**
    * Rounds to an integer number towards 0.
    *
    * @return this number truncated to an integer.
    */
   public IntegerNumber trunc()
   {
      //-------------------------------------
      // PENDING: Reconsider this
      //          This is pretty safe for now
      //-------------------------------------

      BigDecimal bigDecimal = toBigDecimal(0, BigDecimal.ROUND_FLOOR);
      return NumberCentral.valueOf(bigDecimal).trunc();
   }

   //=========================================================================
   // TEXTUAL CONVERSION:
   //
   // - toString
   //
   //=========================================================================

   /**
    * Returns a textual representation of this object.
    *
    * @return a <code>String</code> representation of this object.
    */
   public String toString()
   {
      StringBuffer buffer = new StringBuffer();

      if (elements[0] instanceof CompositeNumber)
      {
         buffer.append('(');
         buffer.append(elements[0]);
         buffer.append(")+");
      }
      else
      {
         buffer.append(elements[0]);
         buffer.append('+');
      }

      if (elements[1] instanceof CompositeNumber)
      {
         buffer.append('(');
         buffer.append(elements[1]);
         buffer.append(')');
      }
      else
      {
         buffer.append(elements[1]);
      }

      return buffer.toString();
   }

}
